/****************************************************************************************** 
 *	Chili DirectX Framework Version 11.12.17											  *	
 *	Game.cpp																			  *
 *	Copyright 2011 PlanetChili.net														  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#include "Game.h"

Game::Game( HWND hWnd,const KeyboardServer& kServer )
:	gfx ( hWnd ),
	kbd( kServer ),
	ScreenWidth( 799 ),
	ScreenHeight( 599 ),

	PaddleWidth( 80 ),
	PaddleX( ScreenWidth/2 - PaddleWidth/2 ),
	
	BallR (10),
	BallX( ScreenWidth/2),
	BallY( ScreenHeight - 45 - BallR ),
	BallDX( 1 ),
	BallDY( 7 ),
	
	r ( 255 ),
	g ( 255 ),
	b ( 255 ),

	Lives(2),
	Moving( false )
	
{
	for(int i = 0; i<55 ; i++)
	{
		BrickState[i] = Filled;
	}
}

void Game::Go()
{
	gfx.BeginFrame();
	PaddleMovement();
	BallMovement();
	NewGame();
	LevelWon();
	ComposeFrame();
	gfx.EndFrame();
}

void Game::PaddleMovement()
{
	if(kbd.LeftIsPressed())
	{
		PaddleX = PaddleX -10;
		if (PaddleX < 15)
		{
			PaddleX = 16;
		}
	}
	
	
	if(kbd.RightIsPressed())
	{
		PaddleX = PaddleX +10;
		if (PaddleX + PaddleWidth > ScreenWidth - 15)
		{
			PaddleX = ScreenWidth - 15 - PaddleWidth;
		}
	}

}

void Game::BallMovement()
{
if (Moving)
	{
		BallY = BallY - BallDY;
		BallX = BallX - BallDX;
		
		//Hit the Wall
		if (BallX - BallDX > ScreenWidth -15 - BallR)
		{
			BallDX = -BallDX;
		}
		if (BallX - BallDX < 15 + BallR)
		{
			BallDX = -BallDX;
		}

		if (BallY - BallDY < 15 + BallR )
		{
			BallDY = -BallDY;
		}

		//Hit the paddle
		if	(BallY - BallDY > ScreenHeight - 45 - BallR 
			&& BallY - BallDY - BallR< ScreenHeight - 45  
			&& BallX + BallR > PaddleX 
			&& BallX - BallR < PaddleX + PaddleWidth)
		{
			BallDY = -BallDY;
			BallDX = -10.0f * (((float)BallX-((float)PaddleX+(float)PaddleWidth/2.0f))/(float)PaddleWidth);
		}

		//Missed the paddle
		if	(BallY + BallR > ScreenHeight - 15) 
		{
			BallDY = 0;
			BallDX = 0;
			BallY= ScreenHeight - 15 - BallR;

			// Restart
			
			if (Lives != 0)
			{
			Lives -= 1;

			PaddleX = ScreenWidth/2 - PaddleWidth/2;
			BallR	= 10;
			BallX	= ScreenWidth/2;
			BallY	= ScreenHeight - 45 - BallR;
			BallDX	= 1;
			BallDY	= 7;
			Moving	= false;
			}
		}
	}
	else
	{
		if(kbd.LeftIsPressed())
		{
			BallX = BallX -10;
			if (BallX < 15 + PaddleWidth/2)
			{
				BallX = 16 + PaddleWidth/2;
			}
		}
		
		if(kbd.RightIsPressed())
		{
			BallX = BallX + 10;
			if (BallX + PaddleWidth/2 > ScreenWidth - 15)
			{
				BallX = ScreenWidth - 15 - PaddleWidth/2;
			}
		}

		if(kbd.SpaceIsPressed())
		{
			Moving = true;
		}

		
	}

	

}

void Game::NewGame()
{
//New Game
	if	(BallY == ScreenHeight - 15 - BallR && kbd.EnterIsPressed() && Lives == 0) 
	{
		Lives = 2;

		PaddleX = ScreenWidth/2 - PaddleWidth/2;
		BallR	= 10;
		BallX	= ScreenWidth/2;
		BallY	= ScreenHeight - 45 - BallR;
		BallDX	= 1;
		BallDY	= 7;
		Moving	= false;

		for(int i = 0; i<55 ; i++)
		{
			BrickState[i] = Filled;
		}
	}
}

void Game::LevelWon()
{
	bool Win = true;
	for(int i = 0; i<55 ; i++)
	{
		if(BrickState[i] == Empty && Win == true)
		{
			Win = true;
		}

		else
		{
			Win = false;
		}
	}

	if (Win)
	{
		Lives = Lives + 1;

		BallDY = BallDY + 2;

		PaddleX = ScreenWidth/2 - PaddleWidth/2;
		BallR	= 10;
		BallX	= ScreenWidth/2;
		BallY	= ScreenHeight - 45 - BallR;
		BallDX	= 1;
		BallDY	= 7;
		Moving	= false;

		for(int i = 0; i<55 ; i++)
		{
			BrickState[i] = Filled;
		}
	}
}

void Game::ComposeFrame()
{
	//Draws a rectangle
	gfx.DrawLine(15,				15,					15,					ScreenHeight - 15,	100,100,100);
	gfx.DrawLine(ScreenWidth - 15,	15,					ScreenWidth -15,	ScreenHeight - 15,	100,100,100);
	gfx.DrawLine(15,				15,					ScreenWidth -15,	15,					100,100,100);
	gfx.DrawLine(15,				ScreenHeight - 15,	ScreenWidth -15,	ScreenHeight - 15,	100,100,100);
	//

	//Draw paddle
	gfx.DrawRect(PaddleX, ScreenHeight-45, PaddleWidth, 10, 255, 255, 255);
	//

	//Draw bricks
	for(int i = 0; i < 5;i++)
	{
		int x ;
		int y = 16 + (27 * i);
		
		//coloring
		switch(i)
		{
			case 0:
				r = 255; b = 0; g = 0;
				break;
			case 1:
				r = 255; b = 0; g = 255;
				break;
			case 2:
				r = 0; b = 255; g = 255;
				break;
			case 3:
				r = 255; b = 255; g = 0;
				break;
			case 4:
				r = 0; b = 0; g = 255;
				break;
			default:
				r = 255; b = 255; g = 255;
				break;
		}

		for(int j = 0;j < 11 ; j++)
		{
			//Hit A brick?
			if(	BallX + BallR > 16 + (70 * j) 		&&
				BallX - BallR < 16 + (70 * j)+ 68	&&
				BallY + BallR > 16 + (27 * i)		&&
				BallY - BallR < 16 + (27 * i)+ 26	)
			{
				if(BrickState[i*11+j] == Filled)
				{	
					BrickState[i*11+j] = Empty;
					BallDY = -BallDY;
				}
			}
			if(BrickState[i*11+j] == Filled)
			{
				x = 16 + (70 * j);
				gfx.DrawRect(x, y, 68, 25, r, b, g);
			}
		}
	}

	//draw ball
	gfx.DrawCircle( BallX, BallY, BallR, 255, 255, 255 );
}